# SPDX-License-Identifier: Apache-2.0.
# Copyright (c) 2024 - 2025 Waldiez and contributors.

# pylint: disable=protected-access,too-many-arguments
# pylint: disable=too-many-positional-arguments
"""Run a waldiez flow.

The flow is first converted to an autogen flow with agents, chats, and tools.
We then chown to temporary directory and:
    either import and call the flow's `main()` (if not isolated),
    or run the flow in a subprocess (if isolated).
Before running the flow, any additional environment
variables specified in the waldiez file are set.
"""

from pathlib import Path
from typing import TYPE_CHECKING, Union

from .models.waldiez import Waldiez
from .running import (
    WaldiezBaseRunner,
    WaldiezImportRunner,
    WaldiezSubprocessRunner,
)

if TYPE_CHECKING:
    from autogen import ChatResult  # type: ignore


class WaldiezRunner(WaldiezBaseRunner):
    """Waldiez runner class."""

    _implementation: WaldiezBaseRunner

    def __init__(
        self,
        waldiez: Waldiez,
        output_path: str | Path | None = None,
        uploads_root: str | Path | None = None,
        structured_io: bool = False,
        isolated: bool = False,
        threaded: bool = False,
        skip_patch_io: bool = True,
    ) -> None:
        """Create a new Waldiez runner.

        Parameters
        ----------
        waldiez : Waldiez
            The waldiez flow to run.
        output_path : str | Path | None, optional
            The path to the output directory where the results will be stored.
            If None, a temporary directory will be used.
        uploads_root : str | Path | None, optional
            The root directory for uploads. If None, the default uploads
            directory will be used.
        structured_io : bool, optional
            If True, the flow will use
            structured IO instead of the default 'input/print'.
        isolated : bool, optional
            If True, the flow will be run in an isolated subprocess.
            Defaults to False.
        threaded : bool, optional
            If True, the flow will be run in a threaded manner.
            Defaults to False.
        skip_patch_io : bool, optional
            If True, the IO patching will be skipped.
            Defaults to True.

        Returns
        -------
        WaldiezBaseRunner
            The runner instance that will execute the flow.
        """
        super().__init__(
            waldiez,
            output_path=output_path,
            uploads_root=uploads_root,
            structured_io=structured_io,
            isolated=isolated,
            threaded=threaded,
            skip_patch_io=skip_patch_io,
        )
        if isolated:
            self._implementation = WaldiezSubprocessRunner(
                waldiez,
                output_path=output_path,
                uploads_root=uploads_root,
                structured_io=structured_io,
                isolated=True,
                threaded=threaded,
                skip_patch_io=skip_patch_io,
            )
        else:
            self._implementation = WaldiezImportRunner(
                waldiez,
                output_path=output_path,
                uploads_root=uploads_root,
                structured_io=structured_io,
                isolated=False,
                threaded=threaded,
                skip_patch_io=skip_patch_io,
            )

    @classmethod
    def load(
        cls,
        waldiez_file: str | Path,
        name: str | None = None,
        description: str | None = None,
        tags: list[str] | None = None,
        requirements: list[str] | None = None,
        output_path: str | Path | None = None,
        uploads_root: str | Path | None = None,
        structured_io: bool = False,
        isolated: bool = False,
        threaded: bool = False,
        skip_patch_io: bool = True,
    ) -> "WaldiezRunner":
        """Load a waldiez flow from a file and create a runner.

        Parameters
        ----------
        waldiez_file : str | Path
            The path to the waldiez file.
        name : str | None, optional
            The name of the flow.
            If None, the name from the waldiez file will be used.
        description : str | None, optional
            The description of the flow.
            If None, the description from the waldiez file will be used.
        tags : list[str] | None, optional
            The tags for the flow. If None, no tags will be set.
        requirements : list[str] | None, optional
            The requirements for the flow. If None, no requirements will be set.
        output_path : str | Path | None, optional
            The path to the output directory where the results will be stored.
            If None, a temporary directory will be used.
        uploads_root : str | Path | None, optional
            The root directory for uploads. If None, the default uploads
            directory will be used.
        structured_io : bool, optional
            If True, the flow will use
            structured IO instead of the default 'input/print'.
        isolated : bool, optional
            If True, the flow will be run in an isolated subprocess.
            Defaults to False.
        threaded : bool, optional
            If True, the flow will be run in a threaded manner.
            Defaults to False.
        skip_patch_io : bool, optional
            If True, the IO patching will be skipped.
            Defaults to True.

        Returns
        -------
        WaldiezBaseRunner
            The runner instance that will execute the flow.
        """
        waldiez = Waldiez.load(
            waldiez_file,
            name=name,
            description=description,
            tags=tags,
            requirements=requirements,
        )
        return cls(
            waldiez,
            output_path=output_path,
            uploads_root=uploads_root,
            structured_io=structured_io,
            isolated=isolated,
            threaded=threaded,
            skip_patch_io=skip_patch_io,
        )

    def _before_run(
        self,
        output_file: Path,
        uploads_root: Path | None,
    ) -> Path:
        """Actions to perform before running the flow.

        Parameters
        ----------
        output_file : Path
            The output file.
        uploads_root : Path | None
            The runtime uploads root.
        """
        return self._implementation._before_run(
            output_file=output_file,
            uploads_root=uploads_root,
        )

    async def _a_before_run(
        self,
        output_file: Path,
        uploads_root: Path | None,
    ) -> Path:
        """Asynchronously perform actions before running the flow.

        Parameters
        ----------
        output_file : str | Path
            The output file.
        uploads_root : Path | None
            The runtime uploads root.

        Returns
        -------
        Path
            The path to the temporary directory created for the run.
        """
        return await self._implementation._a_before_run(
            output_file, uploads_root
        )

    def _run(
        self,
        temp_dir: Path,
        output_file: Path,
        uploads_root: Path | None,
        skip_mmd: bool = False,
    ) -> Union[
        "ChatResult",
        list["ChatResult"],
        dict[int, "ChatResult"],
    ]:
        """Run the flow.

        Parameters
        ----------
        temp_dir : Path
            The path to the temporary directory created for the run.
        output_file : Path
            The output file.
        uploads_root : Path | None
            The runtime uploads root.
        skip_mmd : bool
            Whether to skip generating the mermaid diagram.

        Returns
        -------
        ChatResult | list[ChatResult] | dict[int, ChatResult]
            The result of the run, which can be a single ChatResult,
            a list of ChatResults,
            or a dictionary mapping indices to ChatResults.
        """
        return self._implementation._run(
            temp_dir=temp_dir,
            output_file=output_file,
            uploads_root=uploads_root,
            skip_mmd=skip_mmd,
        )

    async def _a_run(
        self,
        temp_dir: Path,
        output_file: Path,
        uploads_root: Path | None,
        skip_mmd: bool,
    ) -> Union[
        "ChatResult",
        list["ChatResult"],
        dict[int, "ChatResult"],
    ]:  # pyright: ignore
        """Asynchronously run the flow.

        Parameters
        ----------
        temp_dir : Path
            The path to the temporary directory created for the run.
        output_file : Path
            The output file.
        uploads_root : Path | None
            The runtime uploads root.
        skip_mmd : bool
            Whether to skip generating the mermaid diagram.

        Returns
        -------
        Union[ChatResult, list[ChatResult], dict[int, ChatResult]]
            The result of the run, which can be a single ChatResult,
            a list of ChatResults,
            or a dictionary mapping indices to ChatResults.
        """
        return await self._implementation._a_run(
            temp_dir=temp_dir,
            output_file=output_file,
            uploads_root=uploads_root,
            skip_mmd=skip_mmd,
        )

    def _after_run(
        self,
        results: Union[
            "ChatResult",
            list["ChatResult"],
            dict[int, "ChatResult"],
        ],
        output_file: Path,
        uploads_root: Path | None,
        temp_dir: Path,
        skip_mmd: bool,
    ) -> None:
        """Actions to perform after running the flow.

        Parameters
        ----------
        results : Union[ChatResult, list[ChatResult], dict[int, ChatResult]]
            The results of the run, which can be a single ChatResult,
            a list of ChatResults,
            or a dictionary mapping indices to ChatResults.
        output_file : Path
            The path to the output file.
        uploads_root : Path | None
            The runtime uploads root.
        structured_io : bool
            Whether to use structured IO instead of the default 'input/print'.
        temp_dir : Path
            The path to the temporary directory created for the run.
        skip_mmd : bool
            Whether to skip generating the mermaid diagram.
        """
        self._implementation._after_run(
            results,
            output_file,
            uploads_root,
            temp_dir,
            skip_mmd,
        )

    async def _a_after_run(
        self,
        results: Union[
            "ChatResult",
            list["ChatResult"],
            dict[int, "ChatResult"],
        ],
        output_file: Path,
        uploads_root: Path | None,
        temp_dir: Path,
        skip_mmd: bool,
    ) -> None:
        """Asynchronously perform actions after running the flow.

        Parameters
        ----------
        output_file : Path
            The path to the output file.
        uploads_root : Path | None
            The runtime uploads root.
        structured_io : bool
            Whether to use structured IO instead of the default 'input/print'.
        temp_dir : Path
            The path to the temporary directory created for the run.
        skip_mmd : bool
            Whether to skip generating the mermaid diagram.
        """
        await self._implementation._a_after_run(
            results,
            output_file,
            uploads_root,
            temp_dir,
            skip_mmd,
        )

    def start(
        self,
        output_path: str | Path | None,
        uploads_root: str | Path | None,
        structured_io: bool | None = None,
        skip_patch_io: bool | None = None,
        skip_mmd: bool = False,
    ) -> None:
        """Start the flow.

        Parameters
        ----------
        output_path : str | Path | None
            The output path, by default None.
        uploads_root : str | Path | None
            The runtime uploads root.
        structured_io : bool | None
            Whether to use structured IO instead of the default 'input/print'.
            If None, it will use the value from the context.
        skip_patch_io : bool | None = None
            Whether to skip patching I/O, by default None.
            If None, it will use the value from the context.
        skip_mmd : bool = False
            Whether to skip generating the mermaid diagram.
        """
        self._implementation.start(
            output_path=output_path,
            uploads_root=uploads_root,
            structured_io=structured_io,
            skip_patch_io=skip_patch_io,
        )

    def _start(
        self,
        temp_dir: Path,
        output_file: Path,
        uploads_root: Path | None,
        skip_mmd: bool,
    ) -> None:
        """Start the flow.

        Parameters
        ----------
        output_file : Path
            The output file.
        uploads_root : Path | None
            The runtime uploads root.
        skip_mmd : bool
            Whether to skip generating the mermaid diagram.
        """
        self._implementation._start(
            temp_dir=temp_dir,
            output_file=output_file,
            uploads_root=uploads_root,
            skip_mmd=skip_mmd,
        )

    async def a_start(
        self,
        output_path: str | Path | None,
        uploads_root: str | Path | None,
        structured_io: bool | None = None,
        skip_patch_io: bool | None = None,
        skip_mmd: bool = False,
    ) -> None:
        """Asynchronously start the flow.

        Parameters
        ----------
        output_path : str | Path | None
            The output path, by default None.
        uploads_root : str | Path | None
            The runtime uploads root.
        structured_io : bool | None
            Whether to use structured IO instead of the default 'input/print'.
        skip_patch_io : bool | None = None
            Whether to skip patching I/O, by default None.
        skip_mmd : bool = False
            Whether to skip generating the mermaid diagram, by default False.
        """
        await self._implementation.a_start(
            output_path=output_path,
            uploads_root=uploads_root,
            structured_io=structured_io,
            skip_patch_io=skip_patch_io,
            skip_mmd=skip_mmd,
        )

    async def _a_start(
        self,
        temp_dir: Path,
        output_file: Path,
        uploads_root: Path | None,
        skip_mmd: bool,
    ) -> None:
        """Asynchronously start the flow.

        Parameters
        ----------
        temp_dir : Path
            The path to the temporary directory created for the run.
        output_file : Path
            The output file.
        uploads_root : Path | None
            The runtime uploads root.
        skip_mmd : bool
            Whether to skip generating the mermaid diagram.
        """
        await self._implementation._a_start(
            temp_dir=temp_dir,
            output_file=output_file,
            uploads_root=uploads_root,
            skip_mmd=skip_mmd,
        )

    def _stop(self) -> None:
        """Actions to perform when stopping the flow.

        This method should be overridden in subclasses if needed.
        """
        self._implementation._stop()

    async def _a_stop(self) -> None:
        """Asynchronously perform actions when stopping the flow.

        This method should be overridden in subclasses if needed.
        """
        await self._implementation._a_stop()



# SPDX-License-Identifier: Apache-2.0.
# Copyright (c) 2024 - 2025 Waldiez and contributors.
# pylint: disable=line-too-long
# flake8: noqa: E501
"""Predefined Tavily search tool for Waldiez."""

import os
from typing import Any

from ._config import PredefinedToolConfig
from .protocol import PredefinedTool


class TavilySearchToolImpl(PredefinedTool):
    """Tavily search tool for Waldiez."""

    required_secrets: list[str] = ["TAVILY_API_KEY"]
    required_kwargs: dict[str, type] = {}

    @property
    def name(self) -> str:
        """Tool name."""
        return "tavily_search"

    @property
    def description(self) -> str:
        """Tool description."""
        return "Search Tavily for a given query."

    @property
    def kwargs(self) -> dict[str, Any]:
        """Keyword arguments for the tool, used for initialization."""
        return {}

    @property
    def requirements(self) -> list[str]:
        """Python requirements for the tool."""
        return ["ag2[tavily, openai]"]

    @property
    def tags(self) -> list[str]:
        """Tags for the tool, used for categorization."""
        return ["tavily", "search", "web"]

    @property
    def tool_imports(self) -> list[str]:
        """Imports required for the tool implementation."""
        return [
            "from autogen.tools.experimental import TavilySearchTool",
        ]

    def validate_secrets(self, secrets: dict[str, str]) -> list[str]:
        """Validate secrets and return list of missing required ones.

        Parameters
        ----------
        secrets : dict[str, str]
            Dictionary of secrets/environment variables.

        Returns
        -------
        list[str]
            List of missing required secrets.
        """
        missing = []
        if not secrets.get("TAVILY_API_KEY"):
            missing.append("TAVILY_API_KEY")
        return missing

    def validate_kwargs(self, kwargs: dict[str, str]) -> list[str]:
        """Validate keyword arguments and return list of missing required ones.

        Parameters
        ----------
        kwargs : dict[str, str]
            Dictionary of keyword arguments.

        Returns
        -------
        list[str]
            List of missing required keyword arguments.
        """
        return []

    def get_content(
        self,
        secrets: dict[str, str],
    ) -> str:
        """Get content for the tool.

        Parameters
        ----------
        secrets : dict[str, str]
            Dictionary of secrets/environment variables.

        Returns
        -------
        str
            The content for the tool.
        """
        os.environ["TAVILY_API_KEY"] = secrets.get(
            "TAVILY_API_KEY", os.environ.get("TAVILY_API_KEY", "")
        )
        content = f"""
tavily_api_key = os.environ.get("TAVILY_API_KEY", "")
if not tavily_api_key:
    raise ValueError("TAVILY_API_KEY is required for Tavily search tool.")
{self.name} = TavilySearchTool(
    tavily_api_key=tavily_api_key,
)
"""
        return content


TavilySearchTool = TavilySearchToolImpl()
TavilySearchConfig = PredefinedToolConfig(
    name=TavilySearchTool.name,
    description=TavilySearchTool.description,
    required_secrets=TavilySearchTool.required_secrets,
    required_kwargs=TavilySearchTool.required_kwargs,
    requirements=TavilySearchTool.requirements,
    tags=TavilySearchTool.tags,
    implementation=TavilySearchTool,
)



# SPDX-License-Identifier: Apache-2.0.
# Copyright (c) 2024 - 2025 Waldiez and contributors.
"""Predefined Wikipedia search tool for Waldiez."""

from typing import Any

from ._config import PredefinedToolConfig
from .protocol import PredefinedTool


class WikipediaSearchToolImpl(PredefinedTool):
    """Wikipedia search tool for Waldiez."""

    required_secrets: list[str] = []
    kwargs_types: dict[str, type] = {
        "language": str,
        "top_k": int,
        "verbose": bool,
    }
    required_kwargs: dict[str, type] = {}

    @property
    def name(self) -> str:
        """Tool name."""
        return "wikipedia_search"

    @property
    def description(self) -> str:
        """Tool description."""
        return "Search Wikipedia for a given query."

    @property
    def kwargs(self) -> dict[str, Any]:
        """Keyword arguments for the tool, used for initialization."""
        return {
            "language": "en",
            "top_k": 3,
            "verbose": False,
        }

    @property
    def requirements(self) -> list[str]:
        """Python requirements for the tool."""
        return ["ag2[wikipedia, openai]"]

    @property
    def tags(self) -> list[str]:
        """Tags for the tool, used for categorization."""
        return ["wikipedia", "search", "knowledge", "reference"]

    @property
    def tool_imports(self) -> list[str]:
        """Imports required for the tool implementation."""
        return [
            "from autogen.tools.experimental import WikipediaQueryRunTool",
            "from autogen.tools.experimental import WikipediaPageLoadTool",
        ]

    def validate_secrets(self, secrets: dict[str, str]) -> list[str]:
        """Validate secrets and return list of missing required ones.

        Parameters
        ----------
        secrets : dict[str, str]
            Dictionary of secrets/environment variables.

        Returns
        -------
        list[str]
            List of missing required secrets.
        """
        return []

    def validate_kwargs(self, kwargs: dict[str, Any]) -> list[str]:
        """Validate keyword arguments and return list of missing required ones.

        Parameters
        ----------
        kwargs : dict[str, Any]
            Dictionary of keyword arguments.

        Returns
        -------
        list[str]
            List of missing required keyword arguments.
        """
        for key, _ in self.kwargs.items():
            if key in kwargs:
                type_of = self.required_kwargs.get(key, str)
                try:
                    casted = type_of(kwargs[key])
                    if key in self.kwargs:
                        self.kwargs[key] = casted
                except Exception:  # pylint: disable=broad-except
                    pass
        return []

    def get_content(
        self,
        secrets: dict[str, str],
    ) -> str:
        """Get the content of the tool.

        Parameters
        ----------
        secrets : dict[str, str]
            Dictionary of secrets/environment variables.

        Returns
        -------
        str
            Content of the tool.
        """
        content = f"{self.name} = WikipediaQueryRunTool(\n"
        for key, value in self.kwargs.items():
            content += f"    {key}={value!r},\n"
        content += ")\n"
        return content


WikipediaSearchTool = WikipediaSearchToolImpl()
WikipediaSearchConfig = PredefinedToolConfig(
    name=WikipediaSearchTool.name,
    description=WikipediaSearchTool.description,
    required_secrets=WikipediaSearchTool.required_secrets,
    required_kwargs=WikipediaSearchTool.required_kwargs,
    requirements=WikipediaSearchTool.requirements,
    tags=WikipediaSearchTool.tags,
    implementation=WikipediaSearchTool,
)
